@@ -321,7 +321,7 @@ bit_read_RS(Bit_Chain * dat)
   //least significant byte first:
   byte1 = bit_read_RC(dat);
   byte2 = bit_read_RC(dat);
-  return ((uint16_t) ((byte2 << 8) | byte1));
+  return (BITCODE_RS) ((byte2 << 8) | byte1);
 }
 
 /** Read 1 raw short little-endian.
@@ -332,7 +332,7 @@ bit_read_RS_LE(Bit_Chain * dat)
   unsigned char byte1, byte2;
   byte1 = bit_read_RC(dat);
   byte2 = bit_read_RC(dat);
-  return ((uint16_t) ((byte1 << 8) | byte2));
+  return (BITCODE_RS) ((byte1 << 8) | byte2);
 }
 
 /** Write 1 raw short (BE short).
@@ -372,9 +372,10 @@ bit_read_RL(Bit_Chain * dat)
 void
 bit_write_RL(Bit_Chain * dat, BITCODE_RL value)
 {
+  const uint32_t l = value;
   //least significant word first:
-  bit_write_RS(dat, value & 0xFFFF);
-  bit_write_RS(dat, value >> 16);
+  bit_write_RS(dat, l & 0xFFFF);
+  bit_write_RS(dat, l >> 16);
 }
 
 /** Read 1 raw long (4 byte, LE).
@@ -460,26 +461,15 @@ bit_write_RD(Bit_Chain * dat, double value)
 BITCODE_BS
 bit_read_BS(Bit_Chain * dat)
 {
-  unsigned char two_bit_code;
-  unsigned int result;
-
-  two_bit_code = bit_read_BB(dat);
-
+  const unsigned char two_bit_code = bit_read_BB(dat);
   if (two_bit_code == 0)
-    {
-      result = bit_read_RS(dat);
-      return (result);
-    }
+    return bit_read_RS(dat);
   else if (two_bit_code == 1)
-    {
-      result = (unsigned char)bit_read_RC(dat);
-      return (result);
-    }
+    return (BITCODE_BS)bit_read_RC(dat) & 0xFF;
   else if (two_bit_code == 2)
-    return (0);
-  else
-    /* if (two_bit_code == 3) */
-    return (256);
+    return 0;
+  else /* if (two_bit_code == 3) */
+    return 256;
 }
 
 /** Write 1 bitshort (compacted data).
@@ -487,8 +477,9 @@ bit_read_BS(Bit_Chain * dat)
 void
 bit_write_BS(Bit_Chain * dat, BITCODE_BS value)
 {
-
-  if (value > 256)
+  // BITCODE_BS is defined as uint16_t, but better safe than sorry
+  const uint16_t l = value;
+  if (l > 256)
     {
       bit_write_BB(dat, 0);
       bit_write_RS(dat, value);
@@ -509,27 +500,17 @@ bit_write_BS(Bit_Chain * dat, BITCODE_BS value)
 BITCODE_BL
 bit_read_BL(Bit_Chain * dat)
 {
-  unsigned char two_bit_code;
-  BITCODE_BL result;
-
-  two_bit_code = bit_read_BB(dat);
-
+  const unsigned char two_bit_code =  bit_read_BB(dat);
   if (two_bit_code == 0)
-    {
-      result = bit_read_RL(dat);
-      return (result);
-    }
+    return bit_read_RL(dat);
   else if (two_bit_code == 1)
-    {
-      result = bit_read_RC(dat) & 0xFF;
-      return (result);
-    }
+    return bit_read_RC(dat) & 0xFF;
   else if (two_bit_code == 2)
-    return (0);
+    return 0;
   else /* if (two_bit_code == 3) */
     {
       LOG_ERROR("bit_read_BL: unexpected 2-bit code: '11'")
-      return (256);
+      return 256;
     }
 }
 
@@ -538,12 +519,34 @@ bit_read_BL(Bit_Chain * dat)
 void
 bit_write_BL(Bit_Chain * dat, BITCODE_BL value)
 {
-  if (value > 255)
+  // BITCODE_BL is signed int32_t
+  const uint32_t l = value;
+  if (l > 255)
     {
       bit_write_BB(dat, 0);
       bit_write_RL(dat, value);
     }
-  else if (value == 0)
+  else if (l == 0)
+    bit_write_BB(dat, 2);
+  else
+    {
+      bit_write_BB(dat, 1);
+      bit_write_RC(dat, value);
+    }
+}
+
+/** Write signed bitlong (compacted data).
+ */
+void
+bit_write_BLd(Bit_Chain * dat, BITCODE_BLd value)
+{
+  const BITCODE_BL l = value;
+  if (l > 255)
+    {
+      bit_write_BB(dat, 0);
+      bit_write_RL(dat, (BITCODE_RL)value);
+    }
+  else if (l == 0)
     bit_write_BB(dat, 2);
   else
     {
@@ -600,13 +603,19 @@ bit_read_BLL(Bit_Chain * dat)
 {
   unsigned int i, len;
   BITCODE_BLL result = 0ULL;
-  len = bit_read_BB(dat) << 2 | bit_read_B(dat);
-  for (i=0; i<len; i++)
-    {
-      result <<= 8;
-      result |= bit_read_RC(dat);
-    }
-  return result;
+  len = bit_read_BB(dat) << 1 | bit_read_B(dat);
+  switch (len) {
+  case 1: return bit_read_RC(dat);
+  case 2: return bit_read_RS(dat);
+  case 4: return bit_read_RL(dat);
+  default:
+    for (i=0; i<len; i++)
+      {
+        result <<= 8;
+        result |= bit_read_RC(dat);
+      }
+    return result;
+  }
 }
 
 /** Read 1 bitlonglong (compacted uint64_t) as documented (but unused).
@@ -1192,7 +1201,7 @@ bit_read_TF(Bit_Chain *restrict dat, int length)
   bit_read_fixed(dat, chain, length);
   chain[length] = '\0';
 
-  return (char*)chain;
+  return chain;
 }
 
 void
@@ -1200,7 +1209,7 @@ bit_read_fixed(Bit_Chain *restrict dat, char *restrict dest, int length)
 {
   for (int i = 0; i < length; i++)
     {
-      dest[i] = bit_read_RC(dat);
+      dest[i] = (char)bit_read_RC(dat);
     }
 }
 
@@ -1224,6 +1233,7 @@ bit_read_TV(Bit_Chain *restrict dat)
   unsigned char *chain;
 
   length = bit_read_BS(dat);
+  // if (length > AVAIL_BITS()) return DWG_ERR_VALUEOUTOFBOUNDS;
   chain = (unsigned char *) malloc(length + 1);
   for (i = 0; i < length; i++)
     {
@@ -1247,7 +1257,7 @@ bit_write_TV(Bit_Chain *restrict dat, char *restrict chain)
   int i;
   int length;
 
-  length = chain ? strlen((const char *)chain) : 0;
+  length = chain ? strlen((const char *)chain)+1 : 0;
   bit_write_BS(dat, length);
   for (i = 0; i < length; i++)
     bit_write_RC(dat, (unsigned char)chain[i]);
@@ -1285,9 +1295,10 @@ bit_write_TU(Bit_Chain *restrict dat, BITCODE_TU restrict chain)
   if (chain)
     {
 #if defined(HAVE_WCHAR_H) && defined(SIZEOF_WCHAR_T) && SIZEOF_WCHAR_T == 2
-      length = wcslen(chain);
+      length = wcslen(chain)+1;
 #else
       for (length=0; chain[length]; length++) ;
+      length++;
 #endif
     }
   else
@@ -1445,7 +1456,7 @@ bit_write_TIMEBLL(Bit_Chain * dat, BITCODE_TIMEBLL date)
 /** Read color
  */
 void
-bit_read_CMC(Bit_Chain * dat, Dwg_Color* color)
+bit_read_CMC(Bit_Chain *restrict dat, Dwg_Color *restrict color)
 {
   color->index = bit_read_BS(dat);
   if (dat->version >= R_2004)
@@ -1462,7 +1473,7 @@ bit_read_CMC(Bit_Chain * dat, Dwg_Color* color)
 /** Write color
  */
 void
-bit_write_CMC(Bit_Chain * dat, Dwg_Color* color)
+bit_write_CMC(Bit_Chain *restrict dat, Dwg_Color *restrict color)
 {
   bit_write_BS(dat, color->index);
   if (dat->version >= R_2004)
@@ -1476,6 +1487,59 @@ bit_write_CMC(Bit_Chain * dat, Dwg_Color* color)
     }
 }
 
+/** Read entity color (2004+) (truecolor rgb and alpha support)
+ *  Does also references, DBCOLOR lookups (not yet, needs hdl_dat stream)
+ */
+void
+bit_read_EMC(Bit_Chain *restrict dat, Dwg_Color *restrict color)
+{
+  color->index = bit_read_BS(dat);
+  if (dat->version >= R_2004)
+    {
+      uint16_t flag = color->index >> 8;
+      color->index &= 0x1ff;
+      //if (flag & 0x40) {
+      //  color.handle = dwg_decode_handleref(hdl_dat, obj, dwg);
+      //}
+      if (flag & 0x20) {
+        BITCODE_BL alpha = bit_read_BL(dat);
+        //color->transparency_type = trlong & 0xff;
+        //color->alpha = trlong >> 8;
+      }
+      if (!(flag & 0x40) && (flag & 0x80))
+        color->rgb = bit_read_BL(dat);
+      if ((flag & 0x41) == 0x41)
+        color->name = (char*)bit_read_TV(dat); //str_dat?
+      if ((flag & 0x42) == 0x42)
+        color->book_name = (char*)bit_read_TV(dat); //str_dat?
+      color->flag = flag;
+    }
+}
+
+/** Write entity color (2004+)
+ */
+void
+bit_write_EMC(Bit_Chain *restrict dat, Dwg_Color *restrict color)
+{
+  bit_write_BS(dat, (color->index & 0x1ff) | (color->flag << 8));
+  if (dat->version >= R_2004)
+    {
+      uint16_t flag = color->flag;
+      //if (flag & 0x40) {
+      //  color.handle = dwg_decode_handleref(dat, obj, dwg); //hdl_dat!
+      //}
+      if (flag & 0x20) {
+        bit_write_BL(dat, color->alpha);
+      }
+      if (!(flag & 0x40) && (flag & 0x80))
+        bit_write_BL(dat, color->rgb);
+      if ((flag & 0x41) == 0x41)
+        bit_write_TV(dat, color->name); //str_dat
+      if ((flag & 0x42) == 0x42)
+        bit_write_TV(dat, color->book_name); //str_dat
+    }
+}
+
 /** Search for a sentinel; if found, positions "dat->byte" immediately after it
     and returns -1
  */
@@ -1557,6 +1621,44 @@ bit_print(Bit_Chain * dat, long unsigned int size)
   puts("---------------------------------------------------------");
 }
 
+// The i-th bit of a string.
+// 0b1000_0000,0 >> 8 = 1
+#define BIT(b,i) (((b)[(i)/8] & (0x80 >> (i)%8)) >> (7 - (i)%8))
+
+void bit_write_bits(Bit_Chain *restrict dat, const char *restrict bits)
+{
+  char *p = (char *)bits;
+  for (; *p; p++) {
+    bit_write_B(dat, *p != '0');
+  }
+}
+
+void
+bit_print_bits(unsigned char* bits, long unsigned int size)
+{
+  for (long unsigned int i = 0; i < size; i++)
+    {
+      unsigned char bit = i % 8;
+      unsigned char result = (bits[i/8] & (0x80 >> bit)) >> (7 - bit);
+      //if (i && (i % 8 == 0)) printf(" ");
+      printf("%d", result ? 1 : 0);
+    }
+  printf("\n");
+}
+
+void
+bit_fprint_bits(FILE* fp, unsigned char* bits, long unsigned int size)
+{
+  for (long unsigned int i = 0; i < size; i++)
+    {
+      unsigned char bit = i % 8;
+      unsigned char result = (bits[i/8] & (0x80 >> bit)) >> (7 - bit);
+      if (i && !bit) printf(" ");
+      fprintf(fp,"%d", result ? 1 : 0);
+      //fprintf(fp, "%d", BIT(bits, i) ? 1 : 0);
+    }
+}
+
 void
 bit_explore_chain(Bit_Chain * dat, long unsigned int size)
 {
