@@ -715,9 +715,10 @@ dwg_block_control(Dwg_Data *dwg)
 Dwg_Object_Ref*
 dwg_model_space_ref(Dwg_Data *dwg)
 {
-  if (dwg->header_vars.BLOCK_RECORD_MSPACE->obj)
+  if (dwg->header_vars.BLOCK_RECORD_MSPACE && dwg->header_vars.BLOCK_RECORD_MSPACE->obj)
     return dwg->header_vars.BLOCK_RECORD_MSPACE;
-  return dwg->block_control.model_space->obj ? dwg->block_control.model_space : NULL;
+  return dwg->block_control.model_space && dwg->block_control.model_space->obj
+    ? dwg->block_control.model_space : NULL;
 }
 
 /** Returns the paper space block object for the DWG.
@@ -725,9 +726,10 @@ dwg_model_space_ref(Dwg_Data *dwg)
 Dwg_Object_Ref*
 dwg_paper_space_ref(Dwg_Data *dwg)
 {
-  if (dwg->header_vars.BLOCK_RECORD_PSPACE->obj)
+  if (dwg->header_vars.BLOCK_RECORD_PSPACE && dwg->header_vars.BLOCK_RECORD_PSPACE->obj)
     return dwg->header_vars.BLOCK_RECORD_PSPACE;
-  return dwg->block_control.paper_space->obj ? dwg->block_control.paper_space : NULL;
+  return dwg->block_control.paper_space && dwg->block_control.paper_space->obj
+    ? dwg->block_control.paper_space : NULL;
 }
 
 Dwg_Object*
