@@ -94,6 +94,7 @@ static int dxf_read_group(Bit_Chain *dat, int dxf)
   char *endptr;
   long num = strtol((char*)&dat->chain[dat->byte], &endptr, 10);
   if ((int)num == dxf) {
+    LOG_HANDLE("group %d\n", dxf);
     dat->byte += (unsigned char *)endptr - &dat->chain[dat->byte];
     dxf_skip_ws(dat);
     return 1;
@@ -121,7 +122,10 @@ static void dxf_read_string(Bit_Chain *dat, char **string)
   }
 }
 
-static inline void dxf_free_pair(Dxf_Pair* pair)
+#define STRADD(field, string) \
+  field = malloc(strlen(string)+1); strcpy(field, string)
+
+static void dxf_free_pair(Dxf_Pair* pair)
 {
   if (pair->type == VT_STRING ||
       pair->type == VT_BINARY)
@@ -214,7 +218,7 @@ static int dxf_check_code(Bit_Chain *dat, Dxf_Pair *pair, int code)
  */
 
 #define ACTION indxf
-#define IS_ENCODE
+#define IS_ENCODER
 #define IS_DXF
 
 //for sscanf with BD we need to use %lf not %g
@@ -262,6 +266,7 @@ static int dxf_check_code(Bit_Chain *dat, Dxf_Pair *pair, int code)
     char *headername; \
     if (GROUP(9)) { \
       dxf_read_string(dat, &headername); \
+      LOG_TRACE("9 %s:\n", #name); \
       VALUE (value, type, dxf); \
     } \
     else { \
@@ -294,7 +299,7 @@ static int dxf_check_code(Bit_Chain *dat, Dxf_Pair *pair, int code)
 #define RECORD(record) PAIR(0, record)
 #define GROUP(dxf) dxf_read_group(dat, dxf)
 
-#define VALUE(value, type, dxf)
+#define VALUE(value, type, dxf) {}
 
 #define HEADER_HANDLE_NAME(name, dxf, section) \
   HEADER_9(name);\
@@ -326,6 +331,7 @@ static int dxf_check_code(Bit_Chain *dat, Dxf_Pair *pair, int code)
 #define HEADER_RD(name,dxf)  HEADER_9(name); FIELD(name, RD, dxf)
 #define HEADER_RLL(name,dxf) HEADER_9(name); FIELD(name, RLL, dxf)
 #define HEADER_TV(name,dxf)  HEADER_9(name); VALUE_TV(_obj->name,dxf)
+#define HEADER_TU(name,dxf)  HEADER_9(name); VALUE_TU(_obj->name,dxf)
 #define HEADER_T(name,dxf)   HEADER_9(name); VALUE_T(_obj->name, dxf)
 #define HEADER_B(name,dxf)   HEADER_9(name); FIELD(name, B, dxf)
 #define HEADER_BS(name,dxf)  HEADER_9(name); FIELD(name, BS, dxf)
@@ -388,8 +394,9 @@ static int dxf_check_code(Bit_Chain *dat, Dxf_Pair *pair, int code)
 #define FIELD_3BD(name,dxf) {FIELD(name.x, BD, dxf); FIELD(name.y, BD, dxf+10); FIELD(name.z, BD, dxf+20);}
 #define FIELD_3BD_1(name,dxf) {FIELD(name.x, BD, dxf); FIELD(name.y, BD, dxf+1); FIELD(name.z, BD, dxf+2);}
 #define FIELD_3DPOINT(name,dxf) FIELD_3BD(name,dxf)
-#define FIELD_CMC(name,dxf)\
-  VALUE_RS(_obj->name.index, dxf)
+#define FIELD_CMC(color,dxf1,dxf2) \
+  VALUE_RS(_obj->color.index, dxf1)
+// TODO: rgb
 #define FIELD_TIMEBLL(name,dxf) \
   GROUP(dxf);\
   sscanf((char*)&dat->chain[dat->byte], "%u.%u", \
@@ -435,7 +442,7 @@ static int dxf_check_code(Bit_Chain *dat, Dxf_Pair *pair, int code)
 #define FIELD_VECTOR_N(name, type, size, dxf)\
   if (dxf)\
     {\
-      for (vcount=0; vcount < (int)size; vcount++)\
+      for (vcount=0; vcount < (BITCODE_BL)size; vcount++)\
         {\
           sscanf((char*)&dat->chain[dat->byte], #name ": " FORMAT_##type ",\n", \
                  &_obj->name[vcount]);\
@@ -445,10 +452,10 @@ static int dxf_check_code(Bit_Chain *dat, Dxf_Pair *pair, int code)
 #define FIELD_VECTOR_T(name, size, dxf)\
   if (dxf) {\
     PRE (R_2007) {                                                   \
-      for (vcount=0; vcount < (int)_obj->size; vcount++)             \
+      for (vcount=0; vcount < (BITCODE_BL)_obj->size; vcount++)             \
         VALUE_TV(_obj->name[vcount], dxf);                           \
     } else {                                                         \
-      for (vcount=0; vcount < (int)_obj->size; vcount++)             \
+      for (vcount=0; vcount < (BITCODE_BL)_obj->size; vcount++)             \
         VALUE_TU(_obj->name[vcount], dxf);                           \
     }                                                                \
   }
@@ -457,7 +464,7 @@ static int dxf_check_code(Bit_Chain *dat, Dxf_Pair *pair, int code)
 
 #define FIELD_2RD_VECTOR(name, size, dxf)\
   if (dxf) {\
-    for (vcount=0; vcount < (int)_obj->size; vcount++)    \
+    for (vcount=0; vcount < (BITCODE_BL)_obj->size; vcount++)    \
       {\
         FIELD_2RD(name[vcount], dxf);\
       }\
@@ -465,14 +472,14 @@ static int dxf_check_code(Bit_Chain *dat, Dxf_Pair *pair, int code)
 
 #define FIELD_2DD_VECTOR(name, size, dxf)\
   FIELD_2RD(name[0], dxf);\
-  for (vcount = 1; vcount < (int)_obj->size; vcount++)\
+  for (vcount = 1; vcount < (BITCODE_BL)_obj->size; vcount++)\
     {\
       FIELD_2DD(name[vcount], FIELD_VALUE(name[vcount - 1].x), FIELD_VALUE(name[vcount - 1].y), dxf);\
     }\
 
 #define FIELD_3DPOINT_VECTOR(name, size, dxf)\
   if (dxf) {\
-    for (vcount=0; vcount < (int)_obj->size; vcount++)\
+    for (vcount=0; vcount < (BITCODE_BL)_obj->size; vcount++)\
       {\
         FIELD_3DPOINT(name[vcount], dxf);\
       }\
@@ -480,7 +487,7 @@ static int dxf_check_code(Bit_Chain *dat, Dxf_Pair *pair, int code)
 
 #define HANDLE_VECTOR_N(name, size, code, dxf) \
   if (dxf) {\
-    for (vcount=0; vcount < (int)size; vcount++)\
+    for (vcount=0; vcount < (BITCODE_BL)size; vcount++)\
       {\
         FIELD_HANDLE_N(name[vcount], vcount, code, dxf);\
       }\
@@ -530,15 +537,15 @@ static int dxf_check_code(Bit_Chain *dat, Dxf_Pair *pair, int code)
 
 #define DWG_ENTITY(token) \
 static int \
-dwg_indxf_##token (Bit_Chain *dat, Dwg_Object * obj) \
+dwg_indxf_##token (Bit_Chain *restrict dat, Dwg_Object *restrict obj) \
 {\
-  int error = 0;\
-  long vcount, rcount1, rcount2, rcount3, rcount4; \
+  BITCODE_BL vcount, rcount1, rcount2, rcount3, rcount4; \
   Dwg_Entity_##token *ent, *_obj;\
   Bit_Chain *hdl_dat = dat;\
   Dwg_Data* dwg = obj->parent;\
   Dwg_Object_Entity *_ent;\
   Dxf_Pair *pair; \
+  int error = 0;\
   LOG_INFO("Entity " #token ":\n")\
   _ent = obj->tio.entity;\
   _obj = ent = _ent->tio.token;\
@@ -552,14 +559,14 @@ dwg_indxf_##token (Bit_Chain *dat, Dwg_Object * obj) \
 
 #define DWG_OBJECT(token) \
 static int \
-dwg_indxf_ ##token (Bit_Chain *dat, Dwg_Object * obj) \
+dwg_indxf_ ##token (Bit_Chain *restrict dat, Dwg_Object *restrict obj) \
 { \
-  int error = 0; \
-  long vcount, rcount1, rcount2, rcount3, rcount4;\
+  BITCODE_BL vcount, rcount1, rcount2, rcount3, rcount4;\
   Bit_Chain *hdl_dat = dat;\
   Dwg_Data* dwg = obj->parent;\
   Dwg_Object_##token *_obj;\
   Dxf_Pair *pair; \
+  int error = 0; \
   obj->fixedtype = DWG_TYPE_##token;\
   LOG_INFO("Object " #token ":\n")\
   _obj = obj->tio.object->tio.token;\
@@ -692,9 +699,11 @@ dwg_indxf_variable_type(Dwg_Data * dwg, Bit_Chain *dat, Dwg_Object* obj)
 }
 
 static int
-dwg_indxf_object(Bit_Chain *dat, Dwg_Object *obj)
+dwg_indxf_object(Bit_Chain *restrict dat, Dwg_Object *restrict obj)
 {
   int error = 0;
+  if (!obj || !obj->parent)
+    return DWG_ERR_INTERNALERROR;
 
   switch (obj->type)
     {
@@ -835,7 +844,7 @@ dwg_indxf_object(Bit_Chain *dat, Dwg_Object *obj)
     case DWG_TYPE_OLE2FRAME:
       return dwg_indxf_OLE2FRAME(dat, obj);
     case DWG_TYPE_DUMMY:
-      return dwg_indxf_DUMMY(dat, obj);
+      return 0; //dwg_indxf_DUMMY(dat, obj);
     case DWG_TYPE_LONG_TRANSACTION:
       return dwg_indxf_LONG_TRANSACTION(dat, obj);
     case DWG_TYPE_LWPOLYLINE:
@@ -850,6 +859,7 @@ dwg_indxf_object(Bit_Chain *dat, Dwg_Object *obj)
       return dwg_indxf_OLEFRAME(dat, obj);
     case DWG_TYPE_VBA_PROJECT:
       LOG_ERROR("Unhandled Object VBA_PROJECT. Has its own section\n");
+      return DWG_ERR_INVALIDTYPE;
       //dwg_indxf_VBA_PROJECT(dat, obj);
     case DWG_TYPE_LAYOUT:
       return dwg_indxf_LAYOUT(dat, obj);
@@ -907,6 +917,12 @@ dwg_indxf_object(Bit_Chain *dat, Dwg_Object *obj)
   if (dat->byte >= dat->size || \
       (pair->code == 0 && !strcmp(pair->value.s, "ENDSEC"))) \
     break
+#define DXF_RETURN_ENDSEC(what) \
+  if (dat->byte >= dat->size || \
+      (pair->code == 0 && !strcmp(pair->value.s, "ENDSEC"))) { \
+    dxf_free_pair(pair); \
+    return what; \
+  }
 
 static int dxf_expect_code(Bit_Chain *dat, Dxf_Pair *pair, int code)
 {
@@ -941,11 +957,13 @@ dxf_header_read(Bit_Chain *dat, Dwg_Data* dwg)
     pair = dxf_read_pair(dat);
     DXF_BREAK_ENDSEC;
 
-    //TODO process field
-    
+    //TODO find name in header struct and set value
+
     dxf_free_pair(pair);
   }
+  dxf_free_pair(pair);
 
+  // TODO: convert DWGCODEPAGE string to header.codepage number
   if (!strcmp(_obj->DWGCODEPAGE, "ANSI_1252"))
       dwg->header.codepage = 30;
 
@@ -955,24 +973,44 @@ dxf_header_read(Bit_Chain *dat, Dwg_Data* dwg)
 static int
 dxf_classes_read (Bit_Chain *dat, Dwg_Data * dwg)
 {
-  unsigned int i;
+  BITCODE_BL i;
+  Dxf_Pair *pair = dxf_read_pair(dat);
+  Dwg_Class *klass;
 
-  SECTION(CLASSES);
-  for (i=0; i < dwg->num_classes; i++)
-    {
-      RECORD(CLASS);
-      VALUE_TV (dwg->dwg_class[i].dxfname, 1);
-      VALUE_T (dwg->dwg_class[i].cppname, 2);
-      VALUE_T (dwg->dwg_class[i].appname, 3);
-      VALUE_RS (dwg->dwg_class[i].proxyflag, 90);
-      SINCE (R_2004) {
-        VALUE_RC (dwg->dwg_class[i].num_instances, 91);
+  while (1) { // read next class
+    // add class (see decode)
+    i = dwg->num_classes;
+    if (i == 0)
+      dwg->dwg_class = malloc(sizeof(Dwg_Class));
+    else
+      dwg->dwg_class = realloc(dwg->dwg_class, (i + 1) * sizeof(Dwg_Class));
+    if (!dwg->dwg_class) { LOG_ERROR("Out of memory"); return DWG_ERR_OUTOFMEM; }
+
+    klass = &dwg->dwg_class[i];
+    memset(klass, 0, sizeof(Dwg_Class));
+
+    while (pair->code != 0) { // read until next 0 CLASS
+      pair = dxf_read_pair(dat);
+      switch (pair->code) {
+      case 1: STRADD(klass->dxfname, pair->value.s); break;
+      case 2: STRADD(klass->cppname, pair->value.s); break;
+      case 3: STRADD(klass->appname, pair->value.s); break;
+      case 90: klass->proxyflag = pair->value.l; break;
+      case 91: klass->num_instances = pair->value.l; break;
+      case 280: klass->wasazombie = (BITCODE_B)pair->value.i; break;
+      case 281: klass->item_class_id = pair->value.i ? 0x1f3 : 0x1f2; break;
+      default: LOG_WARN("Unknown DXF code for class[%d].%d", i, pair->code);
+               break;
       }
-      VALUE_RC (dwg->dwg_class[i].wasazombie, 280);
-      // Is-an-entity. 1f2 for entities, 1f3 for objects
-      VALUE_RC (dwg->dwg_class[i].item_class_id == 0x1F2 ? 1 : 0, 281);
+      dxf_free_pair(pair);
     }
-  ENDSEC();
+    DXF_RETURN_ENDSEC(0); // next class or ENDSEC
+    if (strcmp(pair->value.s, "CLASS")) { // or something else
+      LOG_ERROR("Unexpexted DXF 0 %s at class[%d]", pair->value.s, i);
+      return DWG_ERR_CLASSESNOTFOUND;
+    }
+    dwg->num_classes++;
+  }
   return 0;
 }
 
@@ -990,7 +1028,7 @@ dxf_tables_read (Bit_Chain *dat, Dwg_Data * dwg)
 }
 
 static int
-dxf_blocks_read (Bit_Chain *dat, Dwg_Data * dwg)
+dxf_blocks_read (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
 {
   (void)dwg;
 
@@ -1001,29 +1039,43 @@ dxf_blocks_read (Bit_Chain *dat, Dwg_Data * dwg)
 }
 
 static int
-dxf_entities_read (Bit_Chain *dat, Dwg_Data * dwg)
+dxf_entities_read (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
 {
+  Dxf_Pair *pair;
+  Dwg_Object *obj = NULL;
   (void)dwg;
 
   SECTION(ENTITIES);
-  //...
+  while (dat->byte < dat->size) {
+    pair = dxf_read_pair(dat);
+    dxf_expect_code(dat, pair, 0);
+    DXF_CHECK_EOF;
+    dwg_indxf_object(dat, obj); //TODO obj must be already created here
+  }
   ENDSEC();
   return 0;
 }
 
 static int
-dxf_objects_read (Bit_Chain *dat, Dwg_Data * dwg)
+dxf_objects_read (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
 {
+  Dxf_Pair *pair;
+  Dwg_Object *obj = NULL;
   (void)dwg;
 
   SECTION(OBJECTS);
-  //...
+  while (dat->byte < dat->size) {
+    pair = dxf_read_pair(dat);
+    dxf_expect_code(dat, pair, 0);
+    DXF_CHECK_EOF;
+    dwg_indxf_object(dat, obj); //TODO obj must be already created here
+  }
   ENDSEC();
   return 0;
 }
 
 static int
-dxf_preview_read (Bit_Chain *dat, Dwg_Data * dwg)
+dxf_preview_read (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
 {
   (void)dwg;
   SECTION(THUMBNAILIMAGE);
@@ -1086,13 +1138,12 @@ dwg_read_dxf(Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
             dxf_free_pair(pair);
             dxf_objects_read (dat, dwg);
           }
+        if (!strcmp(pair->value.s, "THUMBNAIL"))
+          {
+            dxf_free_pair(pair);
+            dxf_preview_read (dat, dwg);
+          }
       }
-    /* if (!strcmp(pair->value.s, "THUMBNAIL"))
-      {
-        dxf_free_pair(pair);
-        dxf_preview_read (dat, dwg);
-      }
-    */
   }
   return dwg->num_objects ? 1 : 0;
 }
